@@ -1,37 +1,54 @@
-#![doc(html_root_url = "https://docs.rs/exmex/0.6.0")]
+#![doc(html_root_url = "https://docs.rs/exmex/0.6.1")]
 //! Exmex is a fast **ex**tendable **m**athematical **ex**pression evaluator.  
 //! ```rust
 //! # use std::error::Error;
 //! # fn main() -> Result<(), Box<dyn Error>> {
 //! #
-//! use exmex::{eval_str};
+//! use exmex::eval_str;
 //! assert!((eval_str("1.5 * ((cos(0) + 23.0) / 2.0)")? - 18.0).abs() < 1e-12);
 //! #
 //! #     Ok(())
 //! # }
 //! ```
-//! For floats, we have a list of predifined operators, namely
-//! `^`, `*`, `/`, `+`, `-`, `sin`, `cos`, `tan`, `exp`, `log`, and `log2`. These are
+//! For floats, we have a list of predifined operators containing
+//! `^`, `*`, `/`, `+`, `-`, `sin`, `cos`, `tan`, `exp`, `log`, and `log2`. The full list is
 //! defined in [`make_default_operators`](make_default_operators).
 //!
 //! ## Variables
-//! For variables we can use curly brackets as shown in the following expression.
+//! For variables we can use strings that are not in the list of operators as shown in the following expression.
+//! Additionally, variables should consist only of letters, numbers, and underscores. More precisely, they need to fit the
+//! regular expression `r"^[a-zA-Z_]+[a-zA-Z_0-9]*"`.
 //! Variables' values are passed as slices to [`eval`](FlatEx::eval).
 //! ```rust
 //! # use std::error::Error;
 //! # fn main() -> Result<(), Box<dyn Error>> {
 //! #
 //! use exmex::{make_default_operators, parse};
-//! let to_be_parsed = "log({x}) + 2* (-{x}^2 + sin(4*{y}))";
-//! let expr = parse::<f64>(to_be_parsed, make_default_operators::<f64>())?;
-//! assert!((expr.eval(&[2.5, 3.7]) - 14.992794866624788 as f64).abs() < 1e-12);
+//! let to_be_parsed = "log(z) + 2* (-z^2 + sin(4*y))";
+//! let expr = parse::<f64>(to_be_parsed, &make_default_operators::<f64>())?;
+//! assert!((expr.eval(&[2.5, 3.7])? - 14.992794866624788 as f64).abs() < 1e-12);
 //! #
 //! #     Ok(())
 //! # }
 //! ```
 //! The `n`-th number in the slice corresponds to the `n`-th variable. Thereby only the
-//! first occurence of the variables is relevant. In this example, we have `x=2.5` and `y=3.7`.
-//!
+//! first occurence of the variables is relevant. In this example, we have `z=2.5` and `y=3.7`.
+//! If variables are between curly brackets, they can have arbitrary names, e.g., 
+//! `{456/549*(}`, `{x}`, and `{x+y}`  are valid variable names as shown in the following.
+//! ```rust
+//! # use std::error::Error;
+//! # fn main() -> Result<(), Box<dyn Error>> {
+//! #
+//! use exmex::{make_default_operators, parse};
+//! let x = 2.1f64;
+//! let y = 0.1f64;
+//! let to_be_parsed = "log({x+y})";  // {x+y} is the name of one(!) variable, not the sum of two 😕.
+//! let expr = parse::<f64>(to_be_parsed, &make_default_operators::<f64>())?;
+//! assert!((expr.eval(&[x+y])? - 2.2f64.ln()).abs() < 1e-12);
+//! #
+//! #     Ok(())
+//! # }
+//! ```
 //! ## Extendability
 //! Library users can also define a different set of operators as shown in the following.
 //! ```rust
@@ -39,7 +56,7 @@
 //! # fn main() -> Result<(), Box<dyn Error>> {
 //! #
 //! use exmex::{parse, BinOp, Operator};
-//! let ops = vec![
+//! let ops = [
 //!     Operator {
 //!         repr: "%",
 //!         bin_op: Some(BinOp{op: |a: i32, b: i32| a % b, prio: 1}),
@@ -51,9 +68,9 @@
 //!         unary_op: None,
 //!     },
 //! ];
-//! let to_be_parsed = "19 % 5 / 2";
-//! let expr = parse::<i32>(to_be_parsed, ops)?;
-//! assert_eq!(expr.eval(&[1, 0]), 2);
+//! let to_be_parsed = "19 % 5 / 2 / a";
+//! let expr = parse::<i32>(to_be_parsed, &ops)?;
+//! assert_eq!(expr.eval(&[1])?, 2);
 //! #
 //! #     Ok(())
 //! # }
@@ -76,16 +93,17 @@
 //! ### Data Types of Numbers
 //!
 //! You can use any type that implements [`Copy`](core::marker::Copy) and
-//! [`FromStr`](std::str::FromStr). In case you do not pass a number that matches the
-//! regex `r"\.?[0-9]+(\.[0-9]+)?"`, you have to pass a suitable regex and use the
-//! function [`parse_with_number_pattern`](parse::parse_with_number_pattern) instead of
+//! [`FromStr`](std::str::FromStr). In case the representation of your data type in the
+//! string does not match the number regex `r"\.?[0-9]+(\.[0-9]+)?"`, you have to pass a
+//! suitable regex and use the function
+//! [`parse_with_number_pattern`](parse::parse_with_number_pattern) instead of
 //! [`parse`](parse::parse). Here is an example for `bool`.
 //! ```rust
 //! # use std::error::Error;
 //! # fn main() -> Result<(), Box<dyn Error>> {
 //! #
 //! use exmex::{parse_with_number_pattern, BinOp, Operator};
-//! let ops = vec![
+//! let ops = [
 //!     Operator {
 //!         repr: "&&",
 //!         bin_op: Some(BinOp{op: |a: bool, b: bool| a && b, prio: 1}),
@@ -103,8 +121,8 @@
 //!     },
 //! ];
 //! let to_be_parsed = "!(true && false) || (!false || (true && false))";
-//! let expr = parse_with_number_pattern::<bool>(to_be_parsed, ops.clone(), "true|false")?;
-//! assert_eq!(expr.eval(&[]), true);
+//! let expr = parse_with_number_pattern::<bool>(to_be_parsed, &ops, "true|false")?;
+//! assert_eq!(expr.eval(&[])?, true);
 //! #
 //! #     Ok(())
 //! # }
@@ -116,29 +134,32 @@
 //! For instance `"---1"` will evalute to `-1`.
 //! If you want to be on the safe side, we suggest using parentheses.
 //!
+//! ## Unicode
+//! Unicode input strings are currently not supported 😕 but might be added in the 
+//! future 😀.
+//!
 
 mod expression;
 mod operators;
 mod parse;
 mod util;
 
-pub use expression::{FlatEx};
+pub use expression::FlatEx;
 
 pub use parse::{parse, parse_with_default_ops, parse_with_number_pattern, ExParseError};
 
 pub use operators::{make_default_operators, BinOp, Operator};
 
-
 /// Parses a string, evaluates a string, and returns the resulting number.
 ///
 /// # Errrors
 ///
-/// In case the parsing went wrong, e.g., due to an invalid input string, an 
+/// In case the parsing went wrong, e.g., due to an invalid input string, an
 /// [`ExParseError`](ExParseError) is returned.
 ///
 pub fn eval_str(text: &str) -> Result<f64, ExParseError> {
     let flat_ex = parse_with_default_ops(text)?;
-    Ok(flat_ex.eval(&vec![]))
+    Ok(flat_ex.eval(&[])?)
 }
 
 #[cfg(test)]
@@ -160,13 +181,13 @@ mod tests {
         fn readme() -> Result<f64, ExParseError> {
             let result = eval_str("sin(73)")?;
             assert_float_eq_f64(result, 73f64.sin());
-            let expr = parse_with_default_ops::<f64>("2*{x}^3-4/{z}")?;
-            let value = expr.eval(&[5.3, 0.5]);
+            let expr = parse_with_default_ops::<f64>("2*x^3-4/z")?;
+            let value = expr.eval(&[5.3, 0.5])?;
             assert_float_eq_f64(value, 289.75399999999996);
             Ok(value)
         }
         fn readme_int() -> Result<u32, ExParseError> {
-            let ops = vec![
+            let ops = [
                 Operator {
                     repr: "|",
                     bin_op: Some(BinOp {
@@ -181,59 +202,114 @@ mod tests {
                     unary_op: Some(|a: u32| !a),
                 },
             ];
-            let expr = parse::<u32>("!({a}|{b})", ops)?;
-            let result = expr.eval(&[0, 1]);
+            let expr = parse::<u32>("!(a|b)", &ops)?;
+            let result = expr.eval(&[0, 1])?;
             assert_eq!(result, u32::MAX - 1);
             Ok(result)
         }
         assert!(!readme().is_err());
         assert!(!readme_int().is_err());
-        
+    }
+    #[test]
+    fn test_variables_curly() {
+        let operators = make_default_operators::<f64>();
+
+        let to_be_parsed = "5*{x} + 4*log2(log(1.5-{gamma}))*({x}*-(tan(cos(sin(652.2-{gamma}))))) + 3*{x}";
+        let expr = parse(to_be_parsed, &operators).unwrap();
+        assert_float_eq_f64(expr.eval(&[1.0, 0.0]).unwrap(), 11.429314405093656);
+        let to_be_parsed = "2*(4*{x} + y^2)";
+        let expr = parse(to_be_parsed, &operators).unwrap();
+        assert_float_eq_f64(expr.eval(&[2.0, 3.0]).unwrap(), 34.0);
+
+        let to_be_parsed = "sin({myvwmlf4i58eo;w/-😕+sin(a)r_25})";
+        let expr = parse(to_be_parsed, &operators).unwrap();
+        assert_float_eq_f64(expr.eval(&[1.5707963267948966]).unwrap(), 1.0);
+
+        let to_be_parsed = "((sin({myvar_25})))";
+        let expr = parse(to_be_parsed, &operators).unwrap();
+        assert_float_eq_f64(expr.eval(&[1.5707963267948966]).unwrap(), 1.0);
     }
     #[test]
     fn test_variables() {
-        let operators = make_default_operators::<f32>();
+        let operators = make_default_operators::<f64>();
+        
+        let to_be_parsed = " -(-(1+x))";
+        let expr = parse(to_be_parsed, &operators).unwrap();
+        assert_float_eq_f64(expr.eval(&[1.0]).unwrap(), 2.0);
 
-        let to_be_parsed = "5*{x} + 4*{y} + 3*{x}";
-        let expr = parse::<f32>(to_be_parsed, operators.clone()).unwrap();
-        assert_float_eq_f32(expr.eval(&[1.0, 0.0]), 8.0);
+        let to_be_parsed = " sin(cos(-3.14159265358979*x))";
+        let expr = parse(to_be_parsed, &operators).unwrap();
+        assert_float_eq_f64(expr.eval(&[1.0]).unwrap(), -0.841470984807896);
 
-        let to_be_parsed = "5*{x} + 4*{y}";
-        let expr = parse::<f32>(to_be_parsed, operators.clone()).unwrap();
-        assert_float_eq_f32(expr.eval(&[0.0, 1.0]), 4.0);
+        let to_be_parsed = "5*sin(x * (4-y^(2-x) * 3 * cos(x-2*(y-1/(y-2*1/cos(sin(x*y))))))*x)";
+        let expr = parse(to_be_parsed, &operators).unwrap();
+        assert_float_eq_f64(expr.eval(&[1.5, 0.2532]).unwrap(), -3.1164569260604176);
 
-        let to_be_parsed = "5*{x} + 4*{y} + {x}^2";
-        let expr = parse::<f32>(to_be_parsed, operators.clone()).unwrap();
-        assert_float_eq_f32(expr.eval(&[2.5, 3.7]), 33.55);
-        assert_float_eq_f32(expr.eval(&[12.0, 9.3]), 241.2);
 
-        let to_be_parsed = "2*(4*{x} + {y}^2)";
-        let expr = parse::<f32>(to_be_parsed, operators.clone()).unwrap();
-        assert_float_eq_f32(expr.eval(&[2.0, 3.0]), 34.0);
+        let to_be_parsed = "5*x + 4*y + 3*x";
+        let expr = parse(to_be_parsed, &operators).unwrap();
+        assert_float_eq_f64(expr.eval(&[1.0, 0.0]).unwrap(), 8.0);
 
-        let to_be_parsed = "sin({myvar_25})";
-        let expr = parse::<f32>(to_be_parsed, operators.clone()).unwrap();
-        assert_float_eq_f32(expr.eval(&[1.5707963267948966]), 1.0);
+        let to_be_parsed = "5*x + 4*y";
+        let expr = parse(to_be_parsed, &operators).unwrap();
+        assert_float_eq_f64(expr.eval(&[0.0, 1.0]).unwrap(), 4.0);
 
-        let to_be_parsed = "((sin({myvar_25})))";
-        let expr = parse::<f32>(to_be_parsed, operators.clone()).unwrap();
-        assert_float_eq_f32(expr.eval(&[1.5707963267948966]), 1.0);
+        let to_be_parsed = "5*x + 4*y + x^2";
+        let expr = parse(to_be_parsed, &operators).unwrap();
+        assert_float_eq_f64(expr.eval(&[2.5, 3.7]).unwrap(), 33.55);
+        assert_float_eq_f64(expr.eval(&[12.0, 9.3]).unwrap(), 241.2);
 
-        let to_be_parsed = "(0 * {myvar_25} + cos({X}))";
-        let expr = parse::<f32>(to_be_parsed, operators.clone()).unwrap();
-        assert_float_eq_f32(
-            expr.eval(&[1.5707963267948966, 3.141592653589793]),
+        let to_be_parsed = "2*(4*x + y^2)";
+        let expr = parse(to_be_parsed, &operators).unwrap();
+        assert_float_eq_f64(expr.eval(&[2.0, 3.0]).unwrap(), 34.0);
+
+        let to_be_parsed = "sin(myvar_25)";
+        let expr = parse(to_be_parsed, &operators).unwrap();
+        assert_float_eq_f64(expr.eval(&[1.5707963267948966]).unwrap(), 1.0);
+
+        let to_be_parsed = "((sin(myvar_25)))";
+        let expr = parse(to_be_parsed, &operators).unwrap();
+        assert_float_eq_f64(expr.eval(&[1.5707963267948966]).unwrap(), 1.0);
+
+        let to_be_parsed = "(0 * myvar_25 + cos(x))";
+        let expr = parse(to_be_parsed, &operators).unwrap();
+        assert_float_eq_f64(
+            expr.eval(&[1.5707963267948966, 3.141592653589793]).unwrap(),
             -1.0,
         );
 
-        let to_be_parsed = "(-{X}^2)";
-        let expr = parse::<f32>(to_be_parsed, operators.clone()).unwrap();
-        assert_float_eq_f32(expr.eval(&[1.0]), 1.0);
+        let to_be_parsed = "(-x^2)";
+        let expr = parse(to_be_parsed, &operators).unwrap();
+        assert_float_eq_f64(expr.eval(&[1.0]).unwrap(), 1.0);
 
-        let to_be_parsed = "log({x}) + 2* (-{x}^2 + sin(4*{y}))";
-        let expr = parse::<f32>(to_be_parsed, operators.clone()).unwrap();
-        assert_float_eq_f32(expr.eval(&[2.5, 3.7]), 14.992794866624788);
+        let to_be_parsed = "log(x) + 2* (-x^2 + sin(4*y))";
+        let expr = parse(to_be_parsed, &operators).unwrap();
+        assert_float_eq_f64(expr.eval(&[2.5, 3.7]).unwrap(), 14.992794866624788);
 
+        let to_be_parsed = "-sqrt(x)/(tanh(5-x)*2) + floor(2.4)* 1/asin(-x^2 + sin(4*sinh(y)))";
+        let expr = parse(to_be_parsed, &operators).unwrap();
+        assert_float_eq_f64(
+            expr.eval(&[2.5, 3.7]).unwrap(),
+            -2.5f64.sqrt() / (2.5f64.tanh() * 2.0)
+                + 2.0 / ((3.7f64.sinh() * 4.0).sin() + 2.5 * 2.5).asin(),
+        );
+
+        let to_be_parsed = "asin(sin(x)) + acos(cos(x)) + atan(tan(x))";
+        let expr = parse(to_be_parsed, &operators).unwrap();
+        assert_float_eq_f64(expr.eval(&[0.5]).unwrap(), 1.5);
+
+        let to_be_parsed = "sqrt(alpha^ceil(centauri))";
+        let expr = parse(to_be_parsed, &operators).unwrap();
+        assert_float_eq_f64(expr.eval(&[2.0, 3.1]).unwrap(), 4.0);
+
+        let to_be_parsed = "trunc(x) + fract(x)";
+        let expr = parse(to_be_parsed, &operators).unwrap();
+        assert_float_eq_f64(expr.eval(&[23422.52345]).unwrap(), 23422.52345);
+
+    }
+
+    #[test]
+    fn test_custom_ops_invert() {
         let ops = vec![
             Operator {
                 repr: "invert",
@@ -246,13 +322,13 @@ mod tests {
                 unary_op: Some(|a: f32| a.sqrt()),
             },
         ];
-        let expr = parse::<f32>("sqrt(invert({a}))", ops).unwrap();
-        assert_float_eq_f32(expr.eval(&[0.25]), 2.0);
+        let expr = parse("sqrt(invert(a))", &ops).unwrap();
+        assert_float_eq_f32(expr.eval(&[0.25]).unwrap(), 2.0);
     }
 
     #[test]
     fn test_custom_ops() {
-        let custom_ops = vec![
+        let custom_ops = [
             Operator {
                 repr: "**",
                 bin_op: Some(BinOp {
@@ -275,8 +351,8 @@ mod tests {
                 unary_op: Some(|a: f32| 1.0 / a),
             },
         ];
-        let expr = parse::<f32>("2**2*invert(3)", custom_ops).unwrap();
-        let val = expr.eval(&vec![]);
+        let expr = parse("2**2*invert(3)", &custom_ops).unwrap();
+        let val = expr.eval(&vec![]).unwrap();
         assert_float_eq_f32(val, 4.0 / 3.0);
 
         let zero_mapper = Operator {
@@ -292,8 +368,8 @@ mod tests {
             .cloned()
             .chain(once(zero_mapper))
             .collect::<Vec<_>>();
-        let expr = parse::<f32>("2^2*1/({berti}) + zer0(4)", extended_operators).unwrap();
-        let val = expr.eval(&[4.0]);
+        let expr = parse("2^2*1/(berti) + zer0(4)", &extended_operators).unwrap();
+        let val = expr.eval(&[4.0]).unwrap();
         assert_float_eq_f32(val, 1.0);
     }
 
@@ -328,10 +404,7 @@ mod tests {
             eval_str(&"-(-1+((-3.14159265358979)/5)*2)").unwrap(),
             2.256637061435916,
         );
-        assert_float_eq_f64(
-            eval_str(&"((2-4)/5)*2").unwrap(),
-            -0.8,
-        );
+        assert_float_eq_f64(eval_str(&"((2-4)/5)*2").unwrap(), -0.8);
         assert_float_eq_f64(
             eval_str(&"-(-1+(sin(-3.14159265358979)/5)*2)").unwrap(),
             1.0,
