@@ -1,16 +1,16 @@
-use crate::expression::{BinOpVec, Expression, FlatEx, Node};
+use crate::expression::{BinOpVec, Expression, FlatEx, Node, N_NODES_ON_STACK};
 use crate::operators::{make_default_operators, BinOp, Operator};
 use crate::util::{apply_unary_ops, CompositionOfUnaryOps};
-use itertools::{izip, Itertools};
+use itertools::Itertools;
+use lazy_static::lazy_static;
 use num::Float;
-use regex::{Regex, RegexSet};
+use regex::Regex;
+use smallvec::SmallVec;
 use std::error::Error;
 use std::fmt::{self, Debug};
 use std::iter::once;
 use std::str::FromStr;
 
-const NUMBER_REGEX_PATTERN: &str = r"\.?[0-9]+(\.[0-9]+)?";
-
 /// This will be thrown at you if the parsing went wrong. Ok, obviously it is not an
 /// exception, so thrown needs to be understood figuratively.
 #[derive(Debug)]
@@ -38,113 +38,130 @@ enum ParsedToken<'a, T: Copy + FromStr> {
     Var(String),
 }
 
+fn is_numeric_text<'a>(text: &'a str) -> Option<&'a str> {
+    let mut n_dots = 0;
+    let n_num_chars = text
+        .chars()
+        .take_while(|c| {
+            let is_dot = *c == '.';
+            if is_dot {
+                n_dots += 1;
+            }
+            c.is_digit(10) || is_dot
+        })
+        .count();
+    if n_num_chars > 0 && n_dots < 2 {
+        Some(&text[0..n_num_chars])
+    } else {
+        None
+    }
+}
+
+fn is_numeric_regex<'a>(re: &Regex, text: &'a str) -> Option<&'a str> {
+    let maybe_num = re.find(text);
+    match maybe_num {
+        Some(m) => Some(m.as_str()),
+        None => None,
+    }
+}
+
 /// Parses tokens of a text with regexes and returns them as a vector
 ///
 /// # Arguments
 ///
 /// * `text` - text to be parsed
-/// * `ops_in` - vector of operator-pairs
-/// * `number_regex_pattern` - defines what in the text will be identified as number
+/// * `ops_in` - slice of operator-pairs
+/// * `is_numeric` - closure that decides whether the current rest of the text starts with a number
 ///
 /// # Errors
 ///
 /// See [`parse_with_number_pattern`](parse_with_number_pattern)
 ///
-fn apply_regexes<'a, T: Copy + FromStr>(
-    text: &str,
-    ops_in: Vec<Operator<'a, T>>,
-    number_regex_pattern: &str,
-) -> Result<Vec<ParsedToken<'a, T>>, ExParseError>
+fn apply_regexes<'a, 'b, T: Copy + FromStr + Debug, F: Fn(&'b str) -> Option<&'b str>>(
+    text: &'b str,
+    ops_in: &[Operator<'a, T>],
+    is_numeric: F,
+) -> Result<SmallVec<[ParsedToken<'a, T>; 2 * N_NODES_ON_STACK]>, ExParseError>
 where
     <T as std::str::FromStr>::Err: Debug,
 {
-    let regex_escapes_ops = r"\|?^*+.";
-
+       
     // We sort operators inverse alphabetically such that log2 has higher priority than log (wlog :D).
-    let mut ops_tmp = ops_in;
+
+    let mut ops_tmp = ops_in.iter().clone().collect::<SmallVec<[_; 64]>>();
     ops_tmp.sort_by(|o1, o2| o2.repr.partial_cmp(o1.repr).unwrap());
     let ops = ops_tmp; // from now on const
 
-    let pattern_ops = ops
-        .iter()
-        .map(|op| {
-            let mut s_tmp = op.repr.to_string();
-            for c in regex_escapes_ops.chars() {
-                s_tmp = s_tmp.replace(c, format!("\\{}", c).as_str());
+    lazy_static! {
+        static ref RE_NAME: Regex = Regex::new(r"^[a-zA-Z_]+[a-zA-Z_0-9]*").unwrap();
+    }
+    
+    let mut cur_offset = 0usize;
+    let find_ops = |offset: usize| {
+        ops.iter().find(|op| {
+            let range_end = offset + op.repr.chars().count();
+            if range_end > text.len() {
+                false
+            } else {
+                op.repr == &text[offset..range_end]
             }
-            s_tmp
         })
-        .collect::<Vec<_>>()
-        .join("|");
-    let pattern_var = r"\{[a-zA-Z_]+[a-zA-Z_0-9]*\}";
-    let pattern_parens = r"\(|\)";
-    let patterns = [
-        pattern_var,
-        pattern_ops.as_str(),
-        number_regex_pattern,
-        pattern_parens,
-    ];
-    let pattern_any = patterns.join("|");
-    let any = match Regex::new(pattern_any.as_str()) {
-        Ok(regex) => regex,
-        Err(_) => {
-            return Err(ExParseError {
-                msg: "Cannot compile the passed number regex.".to_string(),
-            })
-        }
     };
 
-    // Since we checked pattern_any we dare to unwrap.
-    let which_one = RegexSet::new(&patterns).unwrap();
+    let mut res = SmallVec::<[_; 2 * N_NODES_ON_STACK]>::new();
 
-    let captures = any
-        .captures_iter(text)
-        .map(|c| c[0].to_string())
-        .collect::<Vec<_>>();
-
-    let capture_check_string = captures.join("");
-    let unparsed_check = izip!(
-        capture_check_string.chars(),
-        text.chars().filter(|c| *c != ' ')
-    )
-    .find(|(cap, txt)| cap != txt && *txt != ' ');
-
-    match unparsed_check {
-        Some(chars) => Err(ExParseError {
-            msg: format!("unparsed character '{}'", chars.1),
-        }),
-        None => Ok(captures
-            .iter()
-            .map(|elt_string| {
-                let elt_str = elt_string.as_str();
-                let matches = which_one.matches(elt_str);
-                if matches.matched(0) {
-                    ParsedToken::<T>::Var(elt_str[1..elt_str.len() - 1].to_string())
-                } else if matches.matched(1) {
-                    let wrapped_op = ops.iter().find(|op| op.repr == elt_str);
-                    ParsedToken::<T>::Op(match wrapped_op {
-                        Some(op) => *op,
-                        None => {
-                            panic!("This is probably a bug. Could not find operator {}.", elt_str);
-                        }
-                    })
-                } else if matches.matched(2) {
-                    ParsedToken::<T>::Num(elt_str.parse::<T>().unwrap())
-                } else if matches.matched(3) {
-                    let c = elt_str.chars().next().unwrap();
-                    ParsedToken::<T>::Paren(if c == '(' {
-                        Paren::Open
-                    } else if c == ')' {
-                        Paren::Close
-                    } else {
-                        panic!("This is probably a bug. Paren {} is neither ( nor ). Check the paren-regex.", c);
-                    })
-                } else {
-                    panic!("This is probably a bug. Internal regex mismatch!");
-                }
-            })
-            .collect()),
+    for (i, c) in text.chars().enumerate() {
+        if c == ' ' {
+            cur_offset += 1;
+        }
+        if i == cur_offset && cur_offset < text.len() && c != ' ' {
+            let maybe_op;
+            let maybe_num;
+            let maybe_name;
+            let text_rest: &str = &text[cur_offset..];
+            let next_parsed_token = if c == '(' {
+                cur_offset += 1;
+                ParsedToken::<T>::Paren(Paren::Open)
+            } else if c == ')' {
+                cur_offset += 1;
+                ParsedToken::<T>::Paren(Paren::Close)
+            } else if c == '{' {
+                let n_count = text_rest.chars().take_while(|c| *c != '}').count();
+                cur_offset += n_count + 1;
+                ParsedToken::<T>::Var(text_rest[1..n_count + 1].to_string())
+            } else if {
+                maybe_num = is_numeric(text_rest);
+                maybe_num.is_some()
+            } {
+                let num_str = maybe_num.unwrap();
+                let n_chars = num_str.chars().count();
+                cur_offset += n_chars;
+                ParsedToken::<T>::Num(num_str.parse::<T>().unwrap())
+            } else if {
+                maybe_op = find_ops(cur_offset);
+                maybe_op.is_some()
+            } {
+                let op = **maybe_op.unwrap();
+                let n_chars = op.repr.chars().count();
+                cur_offset += n_chars;
+                ParsedToken::<T>::Op(op)
+            } else if {
+                maybe_name = RE_NAME.find(text_rest);
+                maybe_name.is_some()
+            } {
+                let var_str = maybe_name.unwrap().as_str();
+                let n_chars = var_str.chars().count();
+                cur_offset += n_chars;
+                ParsedToken::<T>::Var(maybe_name.unwrap().as_str().to_string())
+            } else {
+                let msg = format!("how to parse the beginning of {}", text_rest);
+                return Err(ExParseError { msg: msg });
+            };
+            res.push(next_parsed_token);
+        }
     }
+
+    Ok(res)
 }
 
 /// Returns an expression that is created recursively and can be evaluated
@@ -263,7 +280,7 @@ where
                             }
                             ParsedToken::Paren(p) => match p {
                                 Paren::Open => {
-                                    let msg = "This is probably a bug An opening paren cannot be the predecessor of a binary opertor";
+                                    let msg = "This is probably a bug. An opening paren cannot be the predecessor of a binary operator.";
                                     panic!("{}", msg);
                                 }
                                 Paren::Close => {
@@ -424,18 +441,41 @@ where
     }
 }
 
+fn parsed_tokens_to_flatex<T: Copy + FromStr + Debug>(
+    parsed_tokens: &SmallVec<[ParsedToken<T>; 2 * N_NODES_ON_STACK]>,
+) -> Result<FlatEx<T>, ExParseError> {
+    let parsed_vars = parsed_tokens
+        .iter()
+        .filter_map(|pt| match pt {
+            ParsedToken::Var(name) => Some(name.clone()),
+            _ => None,
+        })
+        .unique()
+        .collect::<SmallVec<[_; N_NODES_ON_STACK]>>();
+
+    check_preconditions(&parsed_tokens[..])?;
+
+    let (expr, _) = make_expression(
+        &parsed_tokens[0..],
+        &parsed_vars,
+        CompositionOfUnaryOps::new(),
+    )?;
+    Ok(expr.flatten())
+}
+
 /// Parses a string and a vector of operators into an expression that can be evaluated.
 ///
 /// # Errors
 ///
 /// An error is returned in case [`parse_with_number_pattern`](parse_with_number_pattern)
 /// returns one.
-pub fn parse<'a, T>(text: &str, ops: Vec<Operator<'a, T>>) -> Result<FlatEx<T>, ExParseError>
+pub fn parse<'a, T>(text: &str, ops: &[Operator<'a, T>]) -> Result<FlatEx<T>, ExParseError>
 where
     <T as std::str::FromStr>::Err: Debug,
     T: Copy + FromStr + Debug,
 {
-    parse_with_number_pattern::<T>(text, ops, NUMBER_REGEX_PATTERN)
+    let parsed_tokens = apply_regexes(text, ops, is_numeric_text)?;
+    parsed_tokens_to_flatex(&parsed_tokens)
 }
 
 /// Parses a string and a vector of operators and a regex pattern that defines the looks
@@ -469,31 +509,27 @@ where
 /// * in `parsed_tokens` a closing parentheses is directly following an operator, e.g., `+)`, or
 /// * a unary operator is followed directly by a binary operator, e.g., `sin*`.
 ///
-pub fn parse_with_number_pattern<'a, T>(
-    text: &str,
-    ops: Vec<Operator<'a, T>>,
+pub fn parse_with_number_pattern<'a,'b, T>(
+    text: &'b str,
+    ops: &[Operator<'a, T>],
     number_regex_pattern: &str,
 ) -> Result<FlatEx<T>, ExParseError>
 where
     <T as std::str::FromStr>::Err: Debug,
     T: Copy + FromStr + Debug,
 {
-    let parsed_tokens = apply_regexes::<T>(text, ops, number_regex_pattern)?;
-    let parsed_vars = parsed_tokens
-        .iter()
-        .filter_map(|pt| match pt {
-            ParsedToken::Var(name) => Some(name.clone()),
-            _ => None,
-        })
-        .unique()
-        .collect::<Vec<_>>();
-    check_preconditions(&parsed_tokens[..])?;
-    let (expr, _) = make_expression(
-        &parsed_tokens[0..],
-        &parsed_vars,
-        CompositionOfUnaryOps::new(),
-    )?;
-    Ok(expr.flatten())
+    let beginning_number_regex_regex = format!("^({})", number_regex_pattern);
+    let re_number = match Regex::new(beginning_number_regex_regex.as_str()) {
+        Ok(regex) => regex,
+        Err(_) => {
+            return Err(ExParseError {
+                msg: "Cannot compile the passed number regex.".to_string(),
+            })
+        }
+    };
+    let is_numeric = |text: &'b str| is_numeric_regex(&re_number, &text);
+    let parsed_tokens = apply_regexes(text, ops, is_numeric)?;
+    parsed_tokens_to_flatex(&parsed_tokens)
 }
 
 /// Parses a string into an expression that can be evaluated using default operators.
@@ -508,22 +544,18 @@ where
     T: Float + FromStr + Debug,
 {
     let ops = make_default_operators::<T>();
-    // println!("{:#?}", ops);
-    Ok(parse(&text, ops)?)
+    Ok(parse(&text, &ops)?)
 }
 
 #[cfg(test)]
 mod tests {
-    use crate::{
-        parse::{apply_regexes, check_preconditions, make_default_operators, NUMBER_REGEX_PATTERN},
-        ExParseError,
-    };
+    use crate::{ExParseError, parse::{apply_regexes, check_preconditions, is_numeric_text, make_default_operators}};
 
     #[test]
     fn test_apply_regexes() {
         let text = r"5\6";
         let ops = make_default_operators::<f32>();
-        let elts = apply_regexes::<f32>(text, ops, NUMBER_REGEX_PATTERN);
+        let elts = apply_regexes(text, &ops, is_numeric_text);
         assert!(elts.is_err());
     }
 
@@ -540,7 +572,7 @@ mod tests {
                 }
             }
             let ops = make_default_operators::<f32>();
-            let elts = apply_regexes::<f32>(text, ops, NUMBER_REGEX_PATTERN);
+            let elts = apply_regexes(text, &ops, is_numeric_text);
             match elts {
                 Ok(elts_unwr) => {
                     let err = check_preconditions(&elts_unwr[..]);
@@ -553,7 +585,7 @@ mod tests {
         test("", "empty string");
         test("++", "the last element cannot be an operator");
         test(
-            "{a12} (",
+            "a12 (",
             "wlog a number/variable cannot be on the right of a closing paren",
         );
         test("++)", "closing parentheses until");
@@ -562,6 +594,12 @@ mod tests {
         test("12-() ())", "wlog an opening paren");
         test("12-(3-4)*2+ (1/2))", "closing parentheses until");
         test("12-(3-4)*2+ ((1/2)", "parentheses mismatch");
-        test(r"5\6", r"unparsed character '\'");
+        test(r"5\6", r"how to parse the beginning of \");
+        test(r"3 * log2 * 5", r"binary operator cannot be next");
+        test(r"3.4.", r"how to parse the beginning of 3.4.");
+        test(
+            r"3. .4",
+            r"a number/variable cannot be next to a number/variable",
+        );
     }
 }
